

'use strict';


// The ESLint browser environment defines all browser globals as valid,
// even though most people don't know some of them exist (e.g. `name` or `status`).
// This is dangerous as it hides accidentally undefined variables.
// We blacklist the globals that we deem potentially confusing.
// To use them, explicitly reference them, e.g. `window.name` or `window.status`.
const restrictedGlobals = require('confusing-browser-globals');


const path = require('path');
const fs = require('fs');


const appDirectory = fs.realpathSync(process.cwd());
const resolveApp = relativePath => path.resolve(appDirectory, relativePath);
const projectRootPath = resolveApp('.');
const tsConfigPath = resolveApp('tsconfig.json');


module.exports = {
  root: true,
  parser: '@typescript-eslint/parser',
  parserOptions: {
    ecmaVersion: 2018,
    sourceType: 'module',
    ecmaFeatures: {
      jsx: true,
    },

    // typescript-eslint specific options
    project: tsConfigPath,
    tsconfigRootDir: projectRootPath,
    warnOnUnsupportedTypeScriptVersion: true,
  },
  plugins: ['import', 'react-hooks', '@typescript-eslint'],
  extends: ['airbnb', 'plugin:@typescript-eslint/recommended'],
  env: {
    browser: true,
    commonjs: true,
    es6: true,
    jest: true,
    node: true,
  },
  settings: {
    'import/parsers': {
      '@typescript-eslint/parser': ['.ts', '.tsx']
    },
    'import/resolver': {
      // use <root>/tsconfig.json
      typescript: {}
    }
  },
  rules: {

    /**
     * @description rules of eslint official
     */
    /**
     * @bug https://github.com/benmosher/eslint-plugin-import/issues/1282
     * "import/named" temporary disable.
     */
    'import/named': 'off',
    'import/export': 'off', // "import/export" temporary disable.
    'import/prefer-default-export': 'off', // Allow single Named-export
    'no-unused-expressions': [
      'warn',
      {
        allowShortCircuit: true,
        allowTernary: true
      }
    ], // https://eslint.org/docs/rules/no-unused-expressions
    // These ESLint rules are known to cause issues with typescript-eslint
    // See https://github.com/typescript-eslint/typescript-eslint/blob/master/packages/eslint-plugin/src/configs/recommended.json
    camelcase: 'off',
    indent: 'off',
    'no-array-constructor': 'off',
    'no-unused-vars': 'off',
    'no-restricted-globals': ['error'].concat(restrictedGlobals),

    /**
     * @description rules of @typescript-eslint
     */
    '@typescript-eslint/interface-name-prefix': ['warn', 'always'],
    '@typescript-eslint/prefer-interface': 'off', // also want to use "type"
    '@typescript-eslint/explicit-function-return-type': 'warn',
    '@typescript-eslint/camelcase': 'warn', // In this case we don't need this to be error
    '@typescript-eslint/no-use-before-define': 'off', // Redundant
    '@typescript-eslint/no-angle-bracket-type-assertion': 'warn',
    '@typescript-eslint/no-array-constructor': 'warn',
    '@typescript-eslint/no-namespace': 'error',
    '@typescript-eslint/no-unused-vars': [
      'warn',
      {
        args: 'none',
        ignoreRestSiblings: true,
      },
    ],
    '@typescript-eslint/ban-types': [
      'warn',
      {
        types: {
          Object: 'Use {} or Dictionary instead',
          Function: 'Use explicit function type `() => void`',
          String: {
            message: 'Use string instead',
            fixWith: 'string'
          }
        }
      }
    ],

    /**
    * @description rules of eslint-plugin-react
    */
    'react/jsx-filename-extension': [
      'warn',
      {
        extensions: ['.jsx', '.tsx']
      }
    ], // also want to use with ".tsx"
    'react/prop-types': 'off', // Is this incompatible with TS props type?
    /**
    * @description rules of eslint-plugin-react-hooks
    */
    'react-hooks/rules-of-hooks': 'error',
    'react-hooks/exhaustive-deps': 'off' // Hooks doesn't have to have dependencies
  },
}
